==> TConstruct-1.18.2-3.7.2.169-sources.jar: slimeknights/tconstruct/library/json/variable/melee/EntityMeleeVariable.java <==
package slimeknights.tconstruct.library.json.variable.melee;

import com.google.gson.JsonObject;
import net.minecraft.network.FriendlyByteBuf;
import net.minecraft.util.GsonHelper;
import net.minecraft.world.entity.LivingEntity;
import slimeknights.mantle.data.GenericLoaderRegistry.IGenericLoader;
import slimeknights.mantle.data.loader.NestedLoader;
import slimeknights.mantle.util.JsonHelper;
import slimeknights.tconstruct.library.json.variable.entity.EntityVariable;
import slimeknights.tconstruct.library.tools.context.ToolAttackContext;
import slimeknights.tconstruct.library.tools.nbt.IToolStackView;

import javax.annotation.Nullable;
import java.util.Locale;

/**
 * Reads an entity variable from melee content
 * @param entity   Entity variable
 * @param which    Determines whether to read the attacker or the target
 * @param fallback Fallback if the entity is not found
 */
public record EntityMeleeVariable(EntityVariable entity, WhichEntity which, float fallback) implements MeleeVariable {
  @Override
  public float getValue(IToolStackView tool, @Nullable ToolAttackContext context, @Nullable LivingEntity attacker) {
    LivingEntity entity = null;
    if (which == WhichEntity.ATTACKER) {
      entity = attacker;
    } else if (context != null) {
      entity = context.getLivingTarget();
    }
    if (entity != null) {
      return this.entity.getValue(entity);
    }
    return fallback;
  }

  @Override
  public IGenericLoader<? extends MeleeVariable> getLoader() {
    return LOADER;
  }

  public enum WhichEntity { ATTACKER, TARGET }

  public static final IGenericLoader<EntityMeleeVariable> LOADER = new IGenericLoader<>() {
    @Override
    public EntityMeleeVariable deserialize(JsonObject json) {
      NestedLoader.mapType(json, "entity_type");
      return new EntityMeleeVariable(
        EntityVariable.LOADER.deserialize(json),
        JsonHelper.getAsEnum(json, "which", WhichEntity.class),
        GsonHelper.getAsFloat(json, "fallback")
      );
    }

    @Override
    public void serialize(EntityMeleeVariable object, JsonObject json) {
      NestedLoader.serializeInto(json, "entity_type", EntityVariable.LOADER, object.entity);
      json.addProperty("which", object.which.toString().toLowerCase(Locale.ROOT));
      json.addProperty("fallback", object.fallback);
    }

    @Override
    public EntityMeleeVariable fromNetwork(FriendlyByteBuf buffer) {
      return new EntityMeleeVariable(EntityVariable.LOADER.fromNetwork(buffer), buffer.readEnum(WhichEntity.class), buffer.readFloat());
    }

    @Override
    public void toNetwork(EntityMeleeVariable object, FriendlyByteBuf buffer) {
      EntityVariable.LOADER.toNetwork(object.entity, buffer);
      buffer.writeEnum(object.which);
      buffer.writeFloat(object.fallback);
    }
  };
}

==> TConstruct-1.20.1-3.9.1.22-sources.jar: slimeknights/tconstruct/library/json/variable/melee/EntityMeleeVariable.java <==
package slimeknights.tconstruct.library.json.variable.melee;

import net.minecraft.world.entity.LivingEntity;
import slimeknights.mantle.data.loadable.primitive.EnumLoadable;
import slimeknights.mantle.data.loadable.primitive.FloatLoadable;
import slimeknights.mantle.data.loadable.record.RecordLoadable;
import slimeknights.tconstruct.library.json.variable.entity.EntityVariable;
import slimeknights.tconstruct.library.tools.context.ToolAttackContext;
import slimeknights.tconstruct.library.tools.nbt.IToolStackView;

import javax.annotation.Nullable;

/**
 * Reads an entity variable from melee content
 * @param entity   Entity variable
 * @param which    Determines whether to read the attacker or the target
 * @param fallback Fallback if the entity is not found
 */
public record EntityMeleeVariable(EntityVariable entity, WhichEntity which, float fallback) implements MeleeVariable {
  public static final RecordLoadable<EntityMeleeVariable> LOADER = RecordLoadable.create(
    EntityVariable.LOADER.directField("entity_type", EntityMeleeVariable::entity),
    new EnumLoadable<>(WhichEntity.class).requiredField("which", EntityMeleeVariable::which),
    FloatLoadable.ANY.requiredField("fallback", EntityMeleeVariable::fallback),
    EntityMeleeVariable::new);

  @Override
  public float getValue(IToolStackView tool, @Nullable ToolAttackContext context, @Nullable LivingEntity attacker) {
    LivingEntity entity = null;
    if (which == WhichEntity.ATTACKER) {
      entity = attacker;
    } else if (context != null) {
      entity = context.getLivingTarget();
    }
    if (entity != null) {
      return this.entity.getValue(entity);
    }
    return fallback;
  }

  @Override
  public RecordLoadable<EntityMeleeVariable> getLoader() {
    return LOADER;
  }

  public enum WhichEntity { ATTACKER, TARGET }
}
